"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const rokot_apicontroller_1 = require("rokot-apicontroller");
const passport = require("passport");
const knexConnection_1 = require("../data/knexConnection");
const injectTypes_1 = require("../ioc/injectTypes");
const notifications_1 = require("../notification/notifications");
const _ = require("underscore");
const csv_1 = require("../core/csv");
class Middleware {
    static roleProtect(allowedRoleIds) {
        return (req, res, next) => {
            //console.log("roleprotect", req.user, allowedRoles)
            if (!req.isAuthenticated()) {
                res.status(401).send("Unauthenticated");
                return;
            }
            if (allowedRoleIds && allowedRoleIds.length) {
                const profile = req.user;
                if (!profile || !profile.roles || !_.some(profile.roles, r => allowedRoleIds.indexOf(r.roleId) > -1)) {
                    res.status(403).send("Insufficient Access");
                    return;
                }
            }
            next();
        };
    }
}
Middleware.authenticate = passport.authenticate("local");
Middleware.protect = Middleware.roleProtect(null);
Middleware.adminprotect = Middleware.roleProtect([1]);
tslib_1.__decorate([
    rokot_apicontroller_1.api.middlewareProviderFunction("roleprotect", 1),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Array]),
    tslib_1.__metadata("design:returntype", void 0)
], Middleware, "roleProtect", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.middlewareFunction("authenticate"),
    tslib_1.__metadata("design:type", Object)
], Middleware, "authenticate", void 0);
tslib_1.__decorate([
    rokot_apicontroller_1.api.middlewareFunction("protect"),
    tslib_1.__metadata("design:type", Object)
], Middleware, "protect", void 0);
tslib_1.__decorate([
    rokot_apicontroller_1.api.middlewareFunction("adminprotect"),
    tslib_1.__metadata("design:type", Object)
], Middleware, "adminprotect", void 0);
function withTransaction(connection, action, defaultValue) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            return yield connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return yield action(t);
            }));
        }
        catch (error) {
            if (!defaultValue) {
                throw error;
            }
            return defaultValue(error);
        }
    });
}
function sendToNonActivated(connection, nonActivated, notifications, userCreationService, webConfig) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!nonActivated || nonActivated.length == 0) {
            return;
        }
        for (var index = 0; index < nonActivated.length; index++) {
            let profile = nonActivated[index];
            let result = yield withTransaction(connection, (t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let verificationToken = yield userCreationService.createUserVerificationToken(profile.username, t);
                if (!verificationToken) {
                    throw "no token generated";
                }
                let r = yield notifications.notifyNonActivatedNotification({ email: profile.username }, { subject: `${webConfig.siteName}`, token: encodeURIComponent(verificationToken), name: profile.displayName, siteUrl: webConfig.siteUrl });
                if (!r) {
                    throw "no send response";
                }
                if (!r.length) {
                    throw "empty send response";
                }
                if (r[0].error) {
                    throw `error send response: ${JSON.stringify(r[0])}`;
                }
                return true;
            }), (error) => {
                console.log("Error sending activation email", error);
                return false;
            });
            if (!result) {
                console.log(`Unable to create User Verification Token for profile id '${profile ? profile.id : "null"}'`, result);
            }
        }
    });
}
let AuthenticationController = class AuthenticationController {
    constructor(connection, userCreationService, userVerificationService, forgotPasswordService, resetPasswordService, changePasswordService, profileRepository, userRepository, roleRepository, tokenRepository, notifications, webConfig, userConfig) {
        this.connection = connection;
        this.userCreationService = userCreationService;
        this.userVerificationService = userVerificationService;
        this.forgotPasswordService = forgotPasswordService;
        this.resetPasswordService = resetPasswordService;
        this.changePasswordService = changePasswordService;
        this.profileRepository = profileRepository;
        this.userRepository = userRepository;
        this.roleRepository = roleRepository;
        this.tokenRepository = tokenRepository;
        this.notifications = notifications;
        this.webConfig = webConfig;
        this.userConfig = userConfig;
    }
    createUser(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const profileId = yield this.profileRepository.createProfile(req.body.displayName, req.body.username, req.body.metadata, t);
                yield this.roleRepository.updateUserRole(profileId, req.body.roles, t);
                const result = yield this.userCreationService.createUser({ profileId, username: req.body.username }, t);
                this.notifications.completeRegistrationNotification({ email: req.body.username }, { token: encodeURIComponent(result.verificationToken), name: req.body.displayName, siteUrl: this.webConfig.siteUrl });
            }));
            req.sendNoContent();
        });
    }
    createUserReminder(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const user = yield this.userRepository.getUserByUsername(req.body.username, t);
                if (!user) {
                    return;
                }
                const hash = yield this.userCreationService.createUserVerificationToken(req.body.username, t);
                if (!hash) {
                    return;
                }
                const profile = yield this.profileRepository.getProfileById(user["profileId"]);
                this.notifications.completeRegistrationNotification({ email: req.body.username }, { token: encodeURIComponent(hash), name: profile.displayName, siteUrl: this.webConfig.siteUrl });
            }));
            req.sendNoContent();
        });
    }
    rejectUser(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.notifications.rejectRegistrationNotification({ email: req.body.username }, { name: req.body.displayName, siteUrl: this.webConfig.siteUrl });
            }));
            req.sendNoContent();
        });
    }
    activateUsers(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const profiles = yield this.profileRepository.getProfiles();
            const count = parseInt(req.params.count);
            let nonActivated = _.first(profiles.filter(p => !p.isActive && !p.isPendingVerify), count);
            if (this.webConfig.isProduction) {
                sendToNonActivated(this.connection, nonActivated, this.notifications, this.userCreationService, this.webConfig);
            }
            req.sendNoContent();
        });
    }
    updateUser(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const profile = yield this.profileRepository.updateProfile(req.body, true, t);
                req.sendOk(profile);
            }));
        });
    }
    updateProfile(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (req.user.id !== req.body.id) {
                    req.sendForbidden();
                    return;
                }
                const profile = yield this.profileRepository.updateProfile(req.body, false, t);
                req.sendOk(profile);
            }));
        });
    }
    verifyUser(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let action;
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.userVerificationService.verifyUser(req.body, t);
                }
                catch (e) {
                    if (e.message === "User doesn't exist") {
                        action = () => req.sendValidation([{ attribute: "username", message: "Username not pending verification" }]);
                        return;
                    }
                    if (e.message === "User doesn't have a token") {
                        action = () => req.sendValidation([{ attribute: "username", message: "Invalid user verification token" }]);
                        return;
                    }
                    if (e.message === "Invalid verification token") {
                        action = () => req.sendValidation([{ attribute: "username", message: "Invalid verification Token" }]);
                        return;
                    }
                    action = () => req.sendValidation([{ attribute: "username", message: "Unexpected error during verification" }]);
                    return;
                }
                action = () => req.sendNoContent();
            }));
            action && action();
        });
    }
    forgotPassword(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const username = req.body.username;
                try {
                    const user = yield this.userRepository.getUserByUsername(username, t);
                    if (!user) {
                        this.notifications.unknownRegistrationNotification({ email: username }, { name: "user", subject: "Unknown Account Request", operation: "forgotPasswordRequest", siteUrl: this.webConfig.siteUrl });
                        return;
                    }
                    const profile = yield this.profileRepository.getProfileById(user["profileId"]);
                    const actualVerificationTokenHashes = yield this.tokenRepository.getVerificationTokenHashes(username, t);
                    if (actualVerificationTokenHashes.length) {
                        const hash = yield this.userCreationService.createUserVerificationToken(username, t);
                        this.notifications.completeRegistrationNotification({ email: username }, { token: encodeURIComponent(hash), name: profile.displayName, siteUrl: this.webConfig.siteUrl });
                        return;
                    }
                    const result = yield this.forgotPasswordService.forgotPassword(req.body, t);
                    this.notifications.resetPasswordNotification({ email: username }, { token: encodeURIComponent(result.passwordResetToken), name: profile.displayName, siteUrl: this.webConfig.siteUrl });
                }
                catch (e) {
                    if (e.message === "user does not exist") {
                        this.notifications.unknownRegistrationNotification({ email: username }, { name: "user", subject: "Unknown Account Request", operation: "forgotPasswordRequest", siteUrl: this.webConfig.siteUrl });
                        return;
                    }
                    throw e;
                }
            }));
            req.sendNoContent();
        });
    }
    resetPassword(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let action;
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.resetPasswordService.resetPassword(req.body, t);
                }
                catch (e) {
                    if (e.message === "User doesn't exist" || e.message === "User doesn't have a token" || e.message === "Invalid password reset token") {
                        action = () => req.sendValidation([{ attribute: "username", message: "Invalid Username or Reset Token" }]);
                        return;
                    }
                    throw e;
                }
                action = () => req.sendNoContent();
            }));
            action && action();
        });
    }
    changePassword(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const username = yield this.userRepository.getProfileUsername(req.user.id, t);
                try {
                    yield this.changePasswordService.changePassword({ username, oldPassword: req.body.oldPassword, newPassword: req.body.newPassword }, t);
                }
                catch (e) {
                    if (e.message === "Old password doesn't match") {
                        req.sendValidation([{ attribute: "oldPassword", message: "Your existing password is incorrect" }]);
                        return;
                    }
                    throw e;
                }
                req.sendNoContent();
            }));
        });
    }
    login(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this.userRepository.onUserLogin(req.user.username, t);
            }));
            req.sendNoContent();
        });
    }
    logout(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            //req.native.response.cookie("connect.sid", "", { expires: new Date() });
            req.native.request.logout();
            req.sendNoContent();
        });
    }
    profile(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            //console.log("USER:", req.user)
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this.userRepository.onUserLogin(req.user.username, t);
            }));
            req.sendOk(req.user);
        });
    }
    adminProfile(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            //console.log("USER:", req.user)
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this.userRepository.onUserLogin(req.user.username, t);
            }));
            req.sendOk(req.user);
        });
    }
    roles(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const roles = yield this.roleRepository.getRoles(req.body);
            req.sendOk(roles);
        });
    }
    deleteUser(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let deleted = false;
            yield this.connection.knex.transaction((t) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const profileId = req.params.id;
                deleted = yield this.userRepository.deleteUser(profileId, t);
                if (!deleted) {
                    return;
                }
                const dd = yield this.roleRepository.deleteUserRole(profileId, t);
                const df = yield this.profileRepository.deleteProfile(profileId, t);
            }));
            req.sendOk();
        });
    }
    userProfiles(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const profiles = yield this.profileRepository.getProfiles(req.body, req.params);
            req.sendOk(profiles);
        });
    }
    userProfilesCsv(req) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const csvRow = this.userConfig && this.userConfig.csvExportRow;
            const func = csvRow ? (p) => csvRow(p) : (p) => ({ ["Email"]: p.username, ["Name"]: p.displayName });
            const profiles = yield this.profileRepository.getProfiles(req.body, req.params);
            const text = yield csv_1.getCsvText(profiles.map(func));
            req.sendOk(text);
        });
    }
};
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("user"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "createUser", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("user-reminder"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "createUserReminder", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("user-reject"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "rejectUser", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("user-activate/:count"),
    rokot_apicontroller_1.api.verbs("get"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "activateUsers", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("user"),
    rokot_apicontroller_1.api.verbs("put"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "updateUser", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("profile"),
    rokot_apicontroller_1.api.verbs("put"),
    rokot_apicontroller_1.api.middleware("protect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "updateProfile", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("verify"),
    rokot_apicontroller_1.api.verbs("post"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "verifyUser", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("forgot-password"),
    rokot_apicontroller_1.api.verbs("post"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "forgotPassword", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("reset-password"),
    rokot_apicontroller_1.api.verbs("post")
    //@api.middleware("protect")
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "resetPassword", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("change-password"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("protect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "changePassword", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("login"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("authenticate"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "login", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("logout"),
    rokot_apicontroller_1.api.verbs("get"),
    rokot_apicontroller_1.api.middleware("protect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "logout", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("profile"),
    rokot_apicontroller_1.api.verbs("get"),
    rokot_apicontroller_1.api.middleware("protect")
    /** gets a profile */
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "profile", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("admin-profile"),
    rokot_apicontroller_1.api.verbs("get"),
    rokot_apicontroller_1.api.middleware("adminprotect")
    /** gets a profile for an administrator */
    ,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "adminProfile", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("roles"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("protect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "roles", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("profiles/:id"),
    rokot_apicontroller_1.api.verbs("delete"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "deleteUser", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("profiles/:roleId?/:key?/:type?"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "userProfiles", null);
tslib_1.__decorate([
    rokot_apicontroller_1.api.route("profilesCsv/:roleId?/:key?/:type?"),
    rokot_apicontroller_1.api.verbs("post"),
    rokot_apicontroller_1.api.middleware("adminprotect"),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Object]),
    tslib_1.__metadata("design:returntype", Promise)
], AuthenticationController.prototype, "userProfilesCsv", null);
AuthenticationController = tslib_1.__decorate([
    rokot_apicontroller_1.api.controller(injectTypes_1.InjectTypes.AuthenticationController, "auth"),
    tslib_1.__metadata("design:paramtypes", [knexConnection_1.KnexConnection, Object, Object, Object, Object, Object, Object, Object, Object, Object, notifications_1.Notifications, Object, Object])
], AuthenticationController);
exports.AuthenticationController = AuthenticationController;
