/// <reference types="knex" />
import * as knexImport from "knex";
import { KnexConnection } from "./knexConnection";
import { SchemaBuilder } from "./schemaBuilder";
import { IQueryData, IQuerySearch, IQuerySort, IQueryWhere } from '../core/query';
export interface IQueryPage {
    skip?: number;
    take: number;
}
export interface ISelectReturningAll<TTable> extends IQueryBuilderFactory {
    where?: SchemaBuilder.WherePartial<TTable>;
    order?: Partial<Record<keyof TTable, boolean>>;
    tx?: knexImport.Transaction;
    paging?: IQueryPage;
}
export interface ISelect<TTable, TField extends keyof TTable> extends ISelectReturningAll<TTable> {
    fields: TField[];
}
export interface IQueryBuilderFactory {
    factory?: (qb: knexImport.QueryBuilder) => knexImport.QueryBuilder;
}
export interface IUpsertReturningAll<TTable> extends IQueryBuilderFactory {
    data: Partial<TTable>;
    where?: SchemaBuilder.WherePartial<TTable>;
    tx?: knexImport.Transaction;
}
export interface IUpsert<TTable, TField extends keyof TTable> extends IUpsertReturningAll<TTable>, IReturning<TTable, TField> {
}
export interface IReturning<TTable, TField extends keyof TTable> {
    returning: TField[];
}
export interface IDeleteReturningAll<TTable> extends IQueryBuilderFactory {
    where?: SchemaBuilder.WherePartial<TTable>;
    tx?: knexImport.Transaction;
}
export interface IDelete<TTable, TField extends keyof TTable> extends IDeleteReturningAll<TTable>, IReturning<TTable, TField> {
}
export declare class KnexTable<TTable> {
    schema: SchemaBuilder.ISchema<TTable>;
    connection: KnexConnection;
    tableTypeRef: TTable;
    static getIdRowValue<TTable, TCol extends keyof TTable>(table: KnexTable<TTable>, key: TCol, value: TTable[TCol]): TTable[TCol];
    static getIdRowValues<TTable, TCol extends keyof TTable>(table: KnexTable<TTable>, key: TCol, value: TTable[TCol][]): TTable[TCol][];
    static debug: boolean;
    constructor(schema: SchemaBuilder.ISchema<TTable>, connection: KnexConnection);
    private static makeQueryBuilderTx(connection, tableName, tx?);
    makeQueryBuilder(tx?: knexImport.Transaction): knexImport.QueryBuilder;
    insert<TField extends keyof TTable>(update: IUpsert<TTable, TField>): Promise<Pick<TTable, TField>>;
    insert(update: IUpsertReturningAll<TTable>): Promise<number>;
    update<TField extends keyof TTable>(update: IUpsert<TTable, TField>): Promise<Pick<TTable, TField>[]>;
    update(update: IUpsertReturningAll<TTable>): Promise<number>;
    delete<TField extends keyof TTable>(del: IDelete<TTable, TField>): Promise<Pick<TTable, TField>[]>;
    delete(del: IDeleteReturningAll<TTable>): Promise<number>;
    private selectInsert<TField>(qb);
    private selectInsertReturning<TField>(qb);
    private selectFirst<TField>(qb);
    private selectManyItems<TField>(qb);
    select<TField extends keyof TTable>(select: ISelect<TTable, TField>): Promise<Pick<TTable, TField>[]>;
    select(select: ISelectReturningAll<TTable>): Promise<TTable[]>;
    selectOne<TField extends keyof TTable>(select: ISelect<TTable, TField>): Promise<Pick<TTable, TField>>;
    selectOne(select: ISelectReturningAll<TTable>): Promise<TTable>;
    query<TField extends keyof TTable>(select: ISelect<TTable, TField>): knexImport.QueryBuilder;
    query(select: ISelectReturningAll<TTable>): knexImport.QueryBuilder;
    private selectQueryBuilder<TField>(select, finalizer);
    countWhere(where?: SchemaBuilder.WherePartial<TTable>, tx?: knexImport.Transaction, factory?: (qb: knexImport.QueryBuilder) => knexImport.QueryBuilder): Promise<number>;
    maxWhere<TField extends keyof TTable>(field: TField, where: SchemaBuilder.WherePartial<TTable>, tx?: knexImport.Transaction, factory?: (qb: knexImport.QueryBuilder) => knexImport.QueryBuilder): Promise<TTable[TField]>;
    minWhere<TField extends keyof TTable>(field: TField, where: SchemaBuilder.WherePartial<TTable>, tx?: knexImport.Transaction, factory?: (qb: knexImport.QueryBuilder) => knexImport.QueryBuilder): Promise<TTable[TField]>;
    averageWhere<TField extends keyof TTable>(field: TField, where: SchemaBuilder.WherePartial<TTable>, tx?: knexImport.Transaction, factory?: (qb: knexImport.QueryBuilder) => knexImport.QueryBuilder): Promise<TTable[TField]>;
    private scalar<TResponse>(getValue, factoryMain, where?, tx?, factory?);
    private scalarField<TResponse, TField>(field, name, factoryMain, where, tx?, factory?);
    static applyPaging(builder: knexImport.QueryBuilder, paging: IQueryPage): knexImport.QueryBuilder;
    private applyFactory<T>(qb, qbFactory);
    private applyWhere<TTable>(qb, schema, where);
    private applyReturning<T, TTable, TField>(schema, updateData, returning, forceSelectAll);
    private applyOrder<TTable, TField>(qb, schema, order);
}
export interface IDbQueryBuilder {
    connection: KnexConnection;
    applyQueryData(data: IQueryData, builder: knexImport.QueryBuilder): knexImport.QueryBuilder;
    processSearch(jsonField: string, builder: knexImport.QueryBuilder, search: IQuerySearch): knexImport.QueryBuilder;
    processSort(jsonField: string, builder: knexImport.QueryBuilder, sort: IQuerySort): knexImport.QueryBuilder;
    processWhere(jsonField: string, builder: knexImport.QueryBuilder, where: IQueryWhere): knexImport.QueryBuilder;
}
