"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_1 = require("../../core/query");
const _ = require("underscore");
class PostgresQueryBuilder {
    constructor(connection) {
        this.connection = connection;
    }
    applyQueryData(data, qb) {
        if (data) {
            if (data.sort) {
                data.sort.forEach(s => qb = qb.orderBy(s.field, s.descending ? "DESC" : "ASC"));
            }
            if (data.search) {
                const where = data.search.fields.map(f => "(" + data.search.value.split(" ").map(val => `${f} ILIKE '%${val}%'`).join(" AND ") + ")").join(" OR ");
                qb = qb.whereRaw(`(${where})`);
            }
            if (data.where) {
                data.where.forEach(w => {
                    qb = this.whereField(w.field, qb, w);
                });
            }
            if (data.page) {
                qb = this.applyPaging(qb, data.page);
            }
        }
        return qb;
    }
    applyPaging(builder, paging) {
        if (paging) {
            return builder.offset(Math.max(0, paging.skip)).limit(paging.take);
        }
        return builder;
    }
    processSearch(jsonField, builder, search) {
        const where = search.fields.map(f => "(" + search.value.split(" ").map(val => `${jsonField}->>'${f}' ILIKE '%${val}%'`).join(" AND ") + ")").join(" OR ");
        return builder.whereRaw(`(${where})`);
    }
    processSort(jsonField, builder, sort) {
        if (sort.field === "updatedOn") {
            return builder.orderBy("updated_on", sort.descending ? "DESC" : "ASC");
        }
        return builder.orderByRaw(`${jsonField}->>'${sort.field}'${sort.descending ? " DESC" : ""}`);
    }
    whereField(field, builder, where) {
        switch (where.operation) {
            case "in":
                return builder.whereIn(field, query_1.ensureArray(where.value));
            case "neq":
                if (where.value === null) {
                    return builder.whereNotNull(field);
                }
                return builder.whereNot(field, where.value);
            case "eq":
                if (where.value === null) {
                    return builder.whereNull(field);
                }
                return builder.where(field, "=", where.value);
            case "between":
                return builder.whereBetween(field, where.value);
            case "gt":
                return builder.where(field, ">", where.value);
            case "lt":
                return builder.where(field, "<", where.value);
        }
        return builder;
    }
    processWhere(jsonField, builder, where) {
        if (jsonField === "key") {
            return this.whereField(jsonField, builder, where);
        }
        switch (where.operation) {
            case "in":
                return builder.whereRaw(`${jsonField}->>'${where.field}' IN (${this.repeatQueryToken(where.value)})`, where.value);
            case "neq":
                return builder.whereRaw(`${jsonField}->>'${where.field}' != ?`, where.value);
            case "eq":
                return builder.whereRaw(`${jsonField}->>'${where.field}' = ?`, where.value);
            case "between":
                return builder.whereRaw(`${jsonField}->>'${where.field}' BETWEEN ? AND ?`, where.value);
            case "gt":
                return builder.whereRaw(`${jsonField}->>'${where.field}' > ?`, where.value);
            case "lt":
                return builder.whereRaw(`${jsonField}->>'${where.field}' < ?`, where.value);
            // case "array-contains-all":
            //   return builder.whereRaw(`(${jsonField}->>'${where.field}')::jsonb \\?& array[${this.repeatQueryToken(where.values)}]`, where.values)
            // case "array-contains-any":
            //   return builder.whereRaw(`(${jsonField}->>'${where.field}')::jsonb \\?| array[${this.repeatQueryToken(where.values)}]`, where.values)
            case "array-contains-all":
                return builder.whereRaw(`ARRAY(SELECT jsonb_array_elements_text((${jsonField}->>'${where.field}')::jsonb)) @> array[${this.repeatQueryToken(where.value)}]`, where.value);
            case "array-contains-any":
                return builder.whereRaw(`ARRAY(SELECT jsonb_array_elements_text((${jsonField}->>'${where.field}')::jsonb)) && array[${this.repeatQueryToken(where.value)}]`, where.value);
        }
        return builder;
    }
    repeatQueryToken(values) {
        if (_.isArray(values)) {
            return _.range(0, values.length).map(i => "?").join(",");
        }
        else {
            return "?";
        }
    }
}
exports.PostgresQueryBuilder = PostgresQueryBuilder;
