"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const query_1 = require("../../core/query");
const _ = require("underscore");
const knexTable_1 = require("../knexTable");
const queryParamResolver_1 = require("../../queryParams/queryParamResolver");
const propertyPathResolver_1 = require("./propertyPathResolver");
const auth_1 = require("../../core/auth");
const core_1 = require("./core");
function getUniqueValues(fields, values) {
    return _.uniq(_.flatten(fields.map(f => _.flatten(values.map(v => propertyPathResolver_1.PropertyPathResolver.getValue(v, f))))));
}
function modifyUserQuery(currentUserProfile, query, roleRepository) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const profileRoles = currentUserProfile && currentUserProfile.roles && currentUserProfile.roles.length > 0 ? currentUserProfile.roles : [];
        const isAdmin = auth_1.hasAdminRole(profileRoles);
        if (isAdmin) {
            //console.log("ROLE - ALLOW: ADMIN", query.dataTypeKey)
            return query;
        }
        let roles = yield roleRepository.getRolesForDataType(query.dataTypeKey);
        if (!roles) {
            const rolesRelations = yield roleRepository.getRolesRelationsForDataType(query.dataTypeKey);
            if (rolesRelations && rolesRelations.length) {
                let handled = false;
                for (var index = 0; index < rolesRelations.length; index++) {
                    var element = rolesRelations[index];
                    const dataTypeRoles = yield roleRepository.getRolesForDataType(element.dataTypeKey);
                    if (!dataTypeRoles) {
                        continue;
                    }
                    const uu = dataTypeRoles.map(r => profileRoles.find(ur => ur.roleId === r.id)).filter(u => !!u);
                    const keys = _.flatten(uu.map(u => u.dataTypeKeyList));
                    if (!keys || !keys.length) {
                        continue;
                    }
                    if (query_1.isKeyQuery(query)) {
                        const q = Object.assign({}, query, { type: "query", where: [{ field: "key", operation: "eq", value: [query.key] }, { field: element.field, operation: "in", value: keys }] });
                        //console.log("ROLE - ALLOW: MATCHING KEY RESTRICTION", query.dataTypeKey)
                        query = q;
                        handled = true;
                        continue;
                    }
                    //console.log("ROLE - ALLOW: FILTER KEY RESTRICTION", query.dataTypeKey, keys)
                    query.where = query.where || [];
                    query.where.push({ field: element.field, operation: "in", value: keys });
                    handled = true;
                }
                if (handled) {
                    return query;
                }
            }
        }
        if (!roles) {
            //console.log("ROLE - ALLOW: NO RESTRICTION", query.dataTypeKey)
            return query;
        }
        const uu = roles.map(r => profileRoles.find(ur => ur.roleId === r.id)).filter(u => !!u);
        if (uu.length === 0) {
            //console.log("ROLE - FORBID: USER DOESN'T HAVE ROLE", query.dataTypeKey)
            return;
        }
        const keys = _.flatten(uu.map(u => u.dataTypeKeyList));
        if (keys.length === 0) {
            //console.log("ROLE - ALLOW: NO KEY RESTRICTION", query.dataTypeKey)
            return query;
        }
        if (query_1.isKeyQuery(query)) {
            if (keys.indexOf(query.key) === -1) {
                //console.log("ROLE - FORBID: USER DOESN'T HAVE ROLE FOR KEY", query.dataTypeKey)
                //query.key = "____DUMMY____"
                return;
            }
            //console.log("ROLE - ALLOW: MATCHING KEY RESTRICTION", query.dataTypeKey)
            return query;
        }
        //console.log("ROLE - ALLOW: FILTER KEY RESTRICTION", query.dataTypeKey, keys)
        query.where = query.where || [];
        query.where.push({ field: "key", operation: "in", value: keys });
        return query;
    });
}
exports.modifyUserQuery = modifyUserQuery;
function getExcludedDataTypeKeys(releaseConfig) {
    const exclude = releaseConfig && releaseConfig.excludeDataTypeKeys || [];
    exclude.push("upload-file");
    return exclude;
}
class DataRepository {
    constructor(schemas, queryBuilder, roleRepository, releaseConfig) {
        this.schemas = schemas;
        this.queryBuilder = queryBuilder;
        this.roleRepository = roleRepository;
        this.releaseConfig = releaseConfig;
    }
    getReleaseQuery1(tx, factory) {
        return this.schemas.data
            .update({
            factory,
            data: {
                snapshotDataItem: this.schemas.knex.raw(this.schemas.data.schema.columns.dataItem),
                snapshotUpdatedOn: this.schemas.knex.raw(this.schemas.data.schema.columns.updatedOn)
            },
            tx
        });
    }
    release(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.getReleaseQuery1(tx, b => b.whereNotIn(this.schemas.data.schema.columns.dataTypeKey, getExcludedDataTypeKeys(this.releaseConfig)));
        });
    }
    releaseItem(dataTypeKey, dataKey, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const excluded = getExcludedDataTypeKeys(this.releaseConfig);
            if (excluded.indexOf(dataTypeKey) > -1) {
                return;
            }
            yield this.getReleaseQuery1(tx, b => {
                b = b.where(this.schemas.data.schema.columns.dataTypeKey, dataTypeKey);
                if (dataKey) {
                    b = b.andWhere(this.schemas.data.schema.columns.key, dataKey);
                }
                return b;
            });
        });
    }
    delete(query, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.schemas.data.update({ data: { deletedOn: core_1.Dates.now() }, tx, where: { dataTypeKey: query.dataTypeKey, key: query.key } });
        });
    }
    getPending(tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = this.schemas.data;
            const fields = data.schema.columns;
            let b = data.makeQueryBuilder(tx)
                .orderBy(fields.dataTypeKey)
                .orderBy(fields.key)
                .whereNull(fields.deletedOn)
                .whereNotIn(fields.dataTypeKey, getExcludedDataTypeKeys(this.releaseConfig))
                .andWhere(inner => inner.where(fields.updatedOn, ">", this.schemas.knex.raw(`${fields.snapshotUpdatedOn}`)).orWhereNull(fields.snapshotUpdatedOn));
            return yield b.then(d => {
                if (!d || !d.length || !d.map) {
                    return [];
                }
                return d.map(pp => {
                    return { key: pp[fields.key], dataTypeKey: pp[fields.dataTypeKey] };
                });
            });
        });
    }
    get(useSnapshot, currentUserProfile, query, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const all = yield this.getAllData(useSnapshot, currentUserProfile, Object.assign({}, query, { type: "key" }), undefined, tx);
            return all && all.length && all[0];
        });
    }
    getAll(useSnapshot, currentUserProfile, query, page, tx) {
        return this.getAllData(useSnapshot, currentUserProfile, query, page, tx);
    }
    getAllData(useSnapshot, currentUserProfile, query, page, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            query = yield modifyUserQuery(currentUserProfile, query, this.roleRepository);
            if (!query) {
                return [];
            }
            const excludedDataTypeKeys = getExcludedDataTypeKeys(this.releaseConfig);
            const useSnapshotForQuery = useSnapshot && excludedDataTypeKeys.indexOf(query.dataTypeKey) === -1;
            const jsonDataField = useSnapshotForQuery ? "snapshotDataItem" : "dataItem";
            if (query_1.isKeyQuery(query)) {
                const fields = this.schemas.data.schema.columns;
                const docs = yield this.schemas.data.select({
                    where: { dataTypeKey: query.dataTypeKey, key: query.key },
                    fields: ["id", jsonDataField, "updatedOn", "snapshotUpdatedOn"],
                    tx
                });
                const outputDocs = docs.filter(d => !!d).map(d => this.makeDataItem(jsonDataField, query, d));
                return this.loadDataItemReferences(useSnapshot, currentUserProfile, query.references, outputDocs);
            }
            const d = yield this.preProcessQueryWhere(query, useSnapshot, currentUserProfile);
            if (!d) {
                return [];
            }
            let builder = this.schemas.data.select({
                where: { dataTypeKey: query.dataTypeKey, deletedOn: null },
                fields: ["id", jsonDataField, "updatedOn", "snapshotUpdatedOn"],
                factory: b => this.dataQueryFactory(b, this.schemas.data.schema.mapper.toTableColumn(jsonDataField), useSnapshotForQuery, useSnapshot, currentUserProfile, query, page),
                tx
            });
            const docs = yield builder;
            if (query.distinct) {
                const distinctKeysQuery = query.distinctKeysQuery;
                const values = getUniqueValues(query.fields, docs.map(d => d[jsonDataField]));
                if (distinctKeysQuery) {
                    if (!values && !values.length) {
                        return [];
                    }
                    if (!distinctKeysQuery.where) {
                        distinctKeysQuery.where = [];
                    }
                    distinctKeysQuery.where.push({ field: this.schemas.data.schema.columns.key, operation: "in", value: values });
                    return yield this.getAllData(useSnapshot, currentUserProfile, distinctKeysQuery);
                }
                return values;
            }
            const outputDocs = docs.filter(d => !!d).map(d => this.makeDataItem(jsonDataField, query, d));
            return this.loadDataItemReferences(useSnapshot, currentUserProfile, query.references, outputDocs);
        });
    }
    dataQueryFactory(builder, jsonDataField, useSnapshotForQuery, useSnapshot, currentUserProfile, query, page) {
        const fields = this.schemas.data.schema.columns;
        if (useSnapshotForQuery) {
            builder = builder.whereNotNull(jsonDataField);
        }
        if (!query.distinct) {
            builder = knexTable_1.KnexTable.applyPaging(builder, page || query.page);
        }
        if (query.sort && query.sort.length) {
            query.sort.forEach(s => {
                builder = this.queryBuilder.processSort(jsonDataField, builder, s);
            });
        }
        if (query.where && query.where.length) {
            // for (let i = 0; i < query.where.length; i++) {
            //   let w = query.where[i];
            //   if (w.subQuery) {
            //     w.subQuery.paramValues = query.paramValues
            //     let allData = await this.getAllData(useSnapshot, currentUserProfile, w.subQuery);
            //     let uniqueValues = getUniqueValues(w.subQuery.fields, allData)
            //     if (!uniqueValues || !uniqueValues.length) {
            //       return {builder, distinctValues: undefined, distinctKeysQuery: undefined, result: []}
            //     }
            //     w.value = uniqueValues
            //   }
            // }
            if (query.paramValues) {
                queryParamResolver_1.QueryParamResolver.replaceParameters(query.where, query.paramValues);
            }
            query.where.forEach(w => {
                builder = this.queryBuilder.processWhere(jsonDataField, builder, w);
            });
        }
        if (query.search) {
            builder = this.queryBuilder.processSearch(jsonDataField, builder, query.search);
        }
        // const sql = builder.toSQL()
        return builder;
    }
    preProcessQueryWhere(query, useSnapshot, currentUserProfile) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!query) {
                return false;
            }
            if (!query.where || !query.where.length) {
                return true;
            }
            let failed = false;
            for (let i = 0; i < query.where.length; i++) {
                let w = query.where[i];
                if (w.subQuery) {
                    w.subQuery.paramValues = query.paramValues;
                    let allData = yield this.getAllData(useSnapshot, currentUserProfile, w.subQuery);
                    let uniqueValues = getUniqueValues(w.subQuery.fields, allData);
                    if (!uniqueValues || !uniqueValues.length) {
                        failed = true;
                    }
                    w.value = uniqueValues;
                }
            }
            return !failed;
        });
    }
    prepareDataItem(doc, d, dataTypeKey) {
        const dt = d.updatedOn;
        doc[this.schemas.data.schema.columns.id] = d.id;
        doc["updatedOn"] = dt;
        doc["_isReleased"] = d.updatedOn === d.snapshotUpdatedOn;
        doc["_dataTypeKey"] = dataTypeKey;
    }
    makeDataItem(jsonDataField, query, d) {
        let doc = d[jsonDataField];
        if (query.fields) {
            const newDoc = { key: doc["key"] };
            for (let i = 0; i < query.fields.length; i++) {
                let ff = query.fields[i];
                propertyPathResolver_1.PropertyPathResolver.setValue(newDoc, ff, propertyPathResolver_1.PropertyPathResolver.getValue(doc, ff));
            }
            doc = newDoc;
        }
        this.prepareDataItem(doc, d, query.dataTypeKey);
        return doc;
    }
    loadDataItemReferences(useSnapshot, currentUserProfile, references, docs) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (docs.length && references && references.length) {
                const loadReferences = DataReferenceLoader.getLoadReferences(references);
                for (let i = 0; i < loadReferences.length; i++) {
                    let ref = loadReferences[i];
                    let refKey = ref.key;
                    if (!refKey) {
                        continue;
                    }
                    let inMemory = refKey.indexOf("(") === 0;
                    if (inMemory) {
                        refKey = refKey.substr(1, refKey.length - 2);
                    }
                    let ips = refKey.split("|");
                    let propName = ips[0];
                    let dataTypeKey = ips[1];
                    let keys = _.compact(_.flatten(docs.map(d => d[propName])));
                    if (inMemory) {
                        if (ref.references) {
                            yield this.loadDataItemReferences(useSnapshot, currentUserProfile, ref.references, keys);
                        }
                        continue;
                    }
                    yield this.fetchDataItemReferences(useSnapshot, currentUserProfile, dataTypeKey, keys, docs, propName, ref.references);
                }
            }
            return docs;
        });
    }
    fetchDataItemReferences(useSnapshot, currentUserProfile, dataTypeKey, keys, docs, propName, refs) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let q = query_1.createQuery(dataTypeKey);
            let values = _.uniq(keys);
            if (values.length === 0) {
                return;
            }
            if (refs && refs.length > 0) {
                q.references = refs;
            }
            q.where = [{ field: this.schemas.data.schema.columns.key, operation: "in", value: values }];
            try {
                let items = yield this.getAllData(useSnapshot, currentUserProfile, q);
                docs.forEach(d => {
                    const value = d[propName];
                    if (value) {
                        if (_.isArray(value)) {
                            d[`_${propName}Ref`] = _.filter(items, i => value.indexOf(i.key) > -1);
                        }
                        else {
                            d[`_${propName}Ref`] = _.find(items, i => i.key === value);
                        }
                    }
                });
            }
            catch (e) {
            }
        });
    }
    save(params, dataItem, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return dataItem.id ? this.update(params, dataItem, tx) : this.insert(params, dataItem, tx);
        });
    }
    insert(params, dataItem, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const insertData = this.getUpsertData("dataItem", dataItem, params);
            const ins = yield this.schemas.data.insert({ data: insertData, tx, returning: selectDataField("dataItem") });
            const doc = ins.dataItem;
            this.prepareDataItem(doc, ins, params.dataTypeKey);
            return doc;
        });
    }
    update(params, dataItem, tx) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // TODO: Dont delete from passed instance - extend??
            const id = dataItem.id;
            delete dataItem.id;
            const updatedOn = dataItem["updatedOn"];
            const updateData = this.getUpsertData("dataItem", dataItem, params);
            const updated = yield this.schemas.data.update({ data: updateData, tx, where: { id, updatedOn }, returning: selectDataField("dataItem") });
            //const sql = w.toSQL()
            //const docs: any[] = await w
            if (!updated || !updated.length) {
                throw new Error("conflict");
            }
            const up = updated[0];
            const doc = up.dataItem;
            this.prepareDataItem(doc, up, params.dataTypeKey);
            return doc;
        });
    }
    getUpsertData(jsonDataField, dataItem, params) {
        return {
            dataTypeKey: params.dataTypeKey,
            key: dataItem.key.trim(),
            [jsonDataField]: deleteAttachedProperties(dataItem),
            updatedOn: core_1.Dates.now()
        };
    }
}
exports.DataRepository = DataRepository;
function selectDataField(jsonDataField) {
    return ["id", jsonDataField, "updatedOn", "snapshotUpdatedOn"];
    //return [this.dataSchema.data.schema.fields.id, jsonDataField, this.dataSchema.data.schema.fields.updatedOn, this.dataSchema.data.schema.fields.snapshotUpdatedOn]
}
class DataReferenceLoader {
    static getLoadReferences(loadWithItems) {
        const graph = DataReferenceLoader.makeLoadWithGraph(loadWithItems, {});
        return DataReferenceLoader.getLoadWithReference(graph);
    }
    static getLoadWithReference(item) {
        const loadWith = [];
        const keys = Object.keys(item);
        for (let i = 0; i < keys.length; i++) {
            let key = keys[i];
            loadWith.push({ key, references: DataReferenceLoader.loadWithFromKeys(item[key]) });
        }
        return loadWith;
    }
    static loadWithFromKeys(item) {
        const allKeys = [];
        const keys = Object.keys(item);
        for (let i = 0; i < keys.length; i++) {
            let key = keys[i];
            let subKeys = DataReferenceLoader.loadWithFromKeys(item[key]);
            if (subKeys.length) {
                subKeys = subKeys.map(sk => `${key}$${sk}`);
                allKeys.push.apply(allKeys, subKeys);
            }
            else {
                allKeys.push(key);
            }
        }
        return allKeys;
    }
    static makeLoadWithGraph(loadWithItems, addTo) {
        for (let i = 0; i < loadWithItems.length; i++) {
            let item = loadWithItems[i];
            if (!item) {
                return addTo;
            }
            let parts = item.split("$");
            let name = parts.shift();
            let loadWith = parts.join("$");
            addTo[name] = DataReferenceLoader.makeLoadWithGraph([loadWith], addTo[name] || {});
        }
        return addTo;
    }
}
function deleteAttachedProperties(f) {
    for (var key in f) {
        if (key.substring(0, 1) === "_") {
            delete f[key];
        }
    }
    return f;
}
