"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
function selectProfileExtUser(schemas) {
    const user = schemas.user.schema;
    const verificationToken = schemas.verificationToken.schema;
    const passwordResetToken = schemas.passwordResetToken.schema;
    return selectProfileUser(schemas)
        .select(user.qualifiedColumns.passwordHash)
        .select(schemas.knex.raw(`${user.qualifiedColumns.metadata}->>'last_login_at' as last_login_at`))
        .select(schemas.knex(verificationToken.qualifiedName).count().where(verificationToken.qualifiedColumns.userId, schemas.knex.raw(user.qualifiedColumns.id)).as("verify_count"))
        .select(schemas.knex(passwordResetToken.qualifiedName).count().where(passwordResetToken.qualifiedColumns.userId, schemas.knex.raw(user.qualifiedColumns.id)).as("reset_count"));
}
function selectProfileUser(schemas) {
    const user = schemas.user.schema;
    const profile = schemas.profile.schema;
    return schemas.connection.knex(profile.qualifiedName)
        .join(user.qualifiedName, profile.qualifiedColumns.id, "=", user.qualifiedColumns.profileId)
        .select(profile.qualifiedColumns.id, profile.qualifiedColumns.displayName, profile.qualifiedColumns.email, profile.qualifiedColumns.metadata, user.qualifiedColumns.username);
}
function selectProfileIdInRole(schemas, roleKey) {
    const profileRole = schemas.profileRole.schema;
    let q = schemas.connection.knex(profileRole.qualifiedName)
        .select(profileRole.columns.profileId)
        .where(profileRole.columns.roleId, roleKey.roleId);
    if (roleKey.key) {
        if (roleKey.type === "access") {
            return q.andWhereRaw(`(jsonb_array_length(${profileRole.columns.dataTypeKeyList}) = 0 OR ${profileRole.columns.dataTypeKeyList} \\? '${roleKey.key}')`);
        }
        return q.andWhereRaw(`${profileRole.columns.dataTypeKeyList} \\? '${roleKey.key}'`);
    }
    return q;
}
function profileExtFromRow(row, schemas) {
    return Object.assign({ isActive: !!row[schemas.user.schema.columns.passwordHash], isPendingPasswordReset: row["reset_count"] > 0, isPendingVerify: row["verify_count"] > 0, lastLoginAt: row["last_login_at"] }, profileFromRow(row, schemas));
}
function profileFromRow(row, schemas) {
    const user = schemas.user.schema;
    const profile = schemas.profile.schema;
    return {
        id: row[profile.columns.id],
        displayName: row[profile.columns.displayName],
        username: row[user.columns.username],
        email: row[profile.columns.email],
        metadata: row[profile.columns.metadata] // && JSON.parse(row["metadata"]),
    };
}
class ProfileRepository {
    constructor(schemas, queryBuilder, roleRepository) {
        this.schemas = schemas;
        this.queryBuilder = queryBuilder;
        this.roleRepository = roleRepository;
    }
    getProfileById(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield selectProfileUser(this.schemas).where(this.schemas.profile.schema.qualifiedColumns.id, id);
            if (!rows.length) {
                return null;
            }
            const profile = profileFromRow(rows[0], this.schemas);
            profile.roles = yield this.roleRepository.getProfileRoles(profile.id);
            return profile;
        });
    }
    getProfiles(data, roleKey) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let qb = selectProfileExtUser(this.schemas);
            if (roleKey && roleKey.roleId) {
                qb.whereIn(this.schemas.profile.schema.qualifiedColumns.id, selectProfileIdInRole(this.schemas, roleKey));
            }
            const rows = yield this.queryBuilder.applyQueryData(data, qb);
            if (!rows.length) {
                return [];
            }
            const profiles = rows.map(i => profileExtFromRow(i, this.schemas));
            const roles = yield this.roleRepository.getProfileRoleDictionary(profiles.map(p => p.id));
            profiles.forEach(p => {
                const rp = roles[p.id];
                if (rp) {
                    p.roles = rp;
                }
            });
            return profiles;
        });
    }
    deleteProfile(profileId, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.schemas.profile.delete({ where: { id: profileId }, tx: context });
        });
    }
    createProfile(displayName, email, metadata, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield this.schemas.profile.insert({
                data: {
                    displayName,
                    email,
                    metadata
                },
                tx: context,
                returning: ["id"]
            });
            return result.id;
        });
    }
    updateProfile(profile, withRoles, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.schemas.profile.update({
                data: {
                    displayName: profile.displayName,
                    email: profile.email,
                    metadata: profile.metadata,
                },
                returning: ["id"],
                tx: context,
                where: { id: profile.id }
            });
            if (withRoles) {
                profile.roles = yield this.roleRepository.updateUserRole(profile.id, profile.roles, context);
            }
            else {
                profile.roles = yield this.roleRepository.getProfileRoles(profile.id);
            }
            return profile;
        });
    }
}
exports.ProfileRepository = ProfileRepository;
