"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
/**
Get and Set values in JSON data objects
Uses dot notation to access members of the object graph
e.g.
const name = PropertyPathResolver.getValue(person, "name")
const firstAddressTitle = PropertyPathResolver.getValue(person, "addresses.0.title")
*/
class PropertyPathResolver {
    /**
    Get values in JSON data objects using dot notation to access members of the object graph
    e.g.
    const name = PropertyPathResolver.getValue(person, "name")
    const firstAddressTitle = PropertyPathResolver.getValue(person, "addresses.0.title")
    */
    static getValue(data, dataName) {
        const parts = dataName.split(".");
        if (parts.length === 1) {
            return data[dataName];
        }
        return _.reduce(parts, (result, p, index) => {
            if (!result || !p) {
                return;
            }
            const prevIsArrayIterator = index > 0 && parts[index - 1] === "*";
            if (prevIsArrayIterator) {
                return _.flatten(_.without(result.map(r => r[p]), undefined), true);
            }
            const isArrayIterator = p === "*";
            return isArrayIterator ? result : result[p];
        }, data);
    }
    /**
    Set values in JSON data objects using dot notation to access members of the object graph
    e.g.
    PropertyPathResolver.setValue(person, "name", "Dave")
    PropertyPathResolver.setValue(person, "addresses.0.title", "home")
    */
    static setValue(data, dataName, value) {
        const parts = dataName.split(".");
        if (parts.length === 1) {
            data[dataName] = value;
        }
        const lastIndex = parts.length - 1;
        _.each(parts, (p, index) => {
            if (!data || !p) {
                return;
            }
            if (index === lastIndex) {
                data[p] = value;
                return;
            }
            const nextPart = parts[index + 1];
            const isArrayIterator = p === "*";
            if (isArrayIterator) {
                data[0] = { [nextPart]: {} };
                data = data[0];
                return;
            }
            const nextIsArrayIterator = nextPart === "*";
            data[p] = (nextIsArrayIterator || Number(nextPart) !== NaN ? [] : {});
            //console.log(`*** ${p} - ${JSON.stringify(data)} - NEXT ${nextPart} THIS ${p}`)
            data = data[p];
        });
    }
}
exports.PropertyPathResolver = PropertyPathResolver;
