"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const _ = require("underscore");
function toProfileRole(row, schemas) {
    const profileRole = schemas.profileRole.schema;
    return {
        id: row[profileRole.columns.id],
        roleId: row[profileRole.columns.roleId],
        dataTypeKeyList: row[profileRole.columns.dataTypeKeyList]
    };
}
function selectProfileRole(connection, schemas) {
    const profileRole = schemas.profileRole.schema;
    const role = schemas.role.schema;
    return connection.knex(profileRole.qualifiedName)
        .join(role.qualifiedName, profileRole.qualifiedColumns.roleId, `=`, role.qualifiedColumns.id)
        .select(profileRole.qualifiedColumns.id, profileRole.qualifiedColumns.roleId, role.qualifiedColumns.name, role.qualifiedColumns.dataTypeKey, profileRole.qualifiedColumns.profileId, profileRole.qualifiedColumns.dataTypeKeyList);
}
class RoleRepository {
    constructor(schemas, queryBuilder) {
        this.schemas = schemas;
        this.queryBuilder = queryBuilder;
    }
    getRolesForDataType(dataTypeKey) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.role.select({ order: { name: false, id: true }, where: { dataTypeKey } });
            if (!rows.length) {
                return null;
            }
            return rows; //.map(toRole);
        });
    }
    getRolesRelationsForDataType(dataTypeKey) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const roles = yield this.schemas.role.select({});
            if (!roles.length) {
                return null;
            }
            const relatedRoles = roles //.map(toRole)
                .map(r => r.metadata && r.metadata.length && { roleId: r.id, relation: _.find(r.metadata, m => m.dataTypeKey === dataTypeKey) })
                .filter(r => r && !!r.relation);
            return relatedRoles.map(r => ({ field: r.relation.field, roleId: r.roleId, dataTypeKey: r.relation.dataTypeKey }));
        });
    }
    getRoles(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.queryBuilder.applyQueryData(data, this.schemas.role.makeQueryBuilder());
            if (!rows.length) {
                return null;
            }
            return rows.map(r => this.schemas.role.schema.mapper.fromTableRow(r));
        });
    }
    deleteUserRole(profileId, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.schemas.profileRole.delete({ where: { profileId }, tx: context });
        });
    }
    updateUserRole(profileId, roles, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const existingRoleIds = (yield this.schemas.profileRole.select({ fields: ["id"], order: { metadata: true }, where: { profileId }, tx: context })).map(d => d.id);
            const newRoleIds = roles.map(r => r.id);
            const deleteRoleIds = existingRoleIds.filter(er => newRoleIds.indexOf(er) === -1);
            for (let i = 0; i < roles.length; i++) {
                let profileRow = roles[i];
                if (profileRow.id) {
                    yield this.schemas.profileRole.update({
                        data: {
                            roleId: profileRow.roleId,
                            dataTypeKeyList: profileRow.dataTypeKeyList
                        },
                        tx: context,
                        where: { id: profileRow.id }
                    });
                }
                else {
                    const ids = yield this.schemas.profileRole.insert({
                        data: {
                            dataTypeKeyList: profileRow.dataTypeKeyList,
                            profileId,
                            roleId: profileRow.roleId
                        }, tx: context, returning: ["id"]
                    });
                    profileRow.id = ids.id;
                }
            }
            if (deleteRoleIds.length) {
                yield this.schemas.profileRole.makeQueryBuilder(context).delete().whereIn(this.schemas.profileRole.schema.columns.id, deleteRoleIds);
            }
            return roles;
        });
    }
    getRoleById(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield this.schemas.role.select({ where: { id } });
            if (!rows.length) {
                return null;
            }
            return rows[0];
        });
    }
    getProfileRoleDictionary(profileIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield selectProfileRole(this.schemas.profileRole.connection, this.schemas).whereIn(this.schemas.profileRole.schema.qualifiedColumns.profileId, profileIds);
            if (!rows.length) {
                return null;
            }
            const groups = _.groupBy(rows, r => r.profile_id);
            const profileRoles = {};
            _.keys(groups).forEach(g => {
                profileRoles[g] = groups[g].map(o => toProfileRole(o, this.schemas));
            });
            return profileRoles;
        });
    }
    getProfileRoles(profileId) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const rows = yield selectProfileRole(this.schemas.profileRole.connection, this.schemas).where(this.schemas.profileRole.schema.qualifiedColumns.profileId, profileId);
            if (!rows.length) {
                return null;
            }
            return rows.map(i => toProfileRole(i, this.schemas));
        });
    }
    createRole(name, metadata, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield this.schemas.role.insert({
                data: {
                    name: name,
                    metadata: metadata
                },
                tx: context,
                returning: ["id"]
            });
            return result.id;
        });
    }
}
exports.RoleRepository = RoleRepository;
