"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
var SchemaBuilder;
(function (SchemaBuilder) {
    const TableNameKey = "__T_KEY__";
    const tableDecoratedCache = {};
    const columnCache = [];
    function makeColumn(col) {
        return col;
    }
    SchemaBuilder.makeColumn = makeColumn;
    function isRaw(where) {
        return where && !!where.wrap;
    }
    SchemaBuilder.isRaw = isRaw;
    function isQueryBuilder(params) {
        return params && !!params.toSQL;
    }
    SchemaBuilder.isQueryBuilder = isQueryBuilder;
    class ColumnMapper {
        constructor(tableName, columns) {
            this.tableName = tableName;
            this.columns = columns;
            this.propToDbCol = {};
            this.fromTableRow = (db) => {
                const out = {};
                let touched = false;
                db && this.columns.forEach(col => {
                    const value = db[col.name];
                    if (value !== undefined) {
                        touched = true;
                        out[col.propertyName] = col.convertFromDb ? col.convertFromDb(value) : value;
                    }
                });
                return touched ? out : undefined;
            };
            this.toTableRow = (raw, data) => {
                const db = {};
                this.columns.forEach(col => {
                    const value = data[col.propertyName];
                    if (value !== undefined) {
                        if (isQueryBuilder(value) || isRaw(value)) {
                            db[col.name] = value;
                            return;
                        }
                        if (_.isArray(value) && (isQueryBuilder(value[0]) || isRaw(value[0]))) {
                            db[col.name] = value;
                            return;
                        }
                        if (col.sqlToDb) {
                            db[col.name] = col.sqlToDb(raw, col.name, value);
                            return;
                        }
                        if (col.convertToDb) {
                            db[col.name] = col.convertToDb(value);
                            return;
                        }
                        db[col.name] = value;
                    }
                });
                return db;
            };
            this.toTableColumns = (raw, data) => {
                if (!data || !data.length) {
                    return [];
                }
                const mapped = data.map(k => this.propToDbCol[k]).filter(k => !!k);
                if (mapped.length !== data.length) {
                    console.log(`Missing mapping for table '${this.tableName}' Req: '${data.join(", ")}' > '${mapped.map(m => m.name).join(", ")}'`, this.propToDbCol);
                }
                return mapped.map(m => {
                    if (m.sqlFromDb) {
                        const fff = m.sqlFromDb(raw, m.name);
                        return fff; // .toQuery()
                    }
                    return m.name;
                });
            };
            this.toTableColumn = (data) => {
                if (!data) {
                    return;
                }
                return this.propToDbCol[data].name;
            };
            columns.forEach(k => {
                this.propToDbCol[k.propertyName] = k;
            });
        }
    }
    function qualify(parent, child) {
        if (!parent) {
            return child;
        }
        return `${parent}.${child}`;
    }
    function createTableSchemas(baseSchema, item) {
        function tableSchemas() {
            const sr = {};
            Object.keys(item).forEach((key) => {
                sr[key] = createTableSchema(baseSchema, item[key]);
            });
            //console.log(JSON.stringify(sr, null, 1))
            return sr;
        }
        return tableSchemas();
    }
    SchemaBuilder.createTableSchemas = createTableSchemas;
    const tableCache = {};
    function createTableSchema(baseSchema, ctor) {
        const name = ctor[TableNameKey];
        if (tableCache[name]) {
            return tableCache[name];
        }
        const tableDecorated = tableDecoratedCache[name];
        const qualifiedName = qualify(baseSchema, name);
        const mapper = new ColumnMapper(name, tableDecorated.columns.map(i => toColumn(i, qualify(qualifiedName, i.column.name))));
        const qualifiedColumns = {};
        const columns = {};
        const defs = {};
        const allKeys = [];
        const primaryKeys = [];
        const primaryKeyKinds = [];
        mapper.columns.forEach((def) => {
            const pn = def.propertyName;
            qualifiedColumns[pn] = def.qualifiedName;
            defs[pn] = {
                fromRow(row) {
                    const db = row[def.name];
                    return def.convertFromDb ? def.convertFromDb(db) : db;
                },
                toRowData(value) {
                    return def.convertToDb ? def.convertToDb(value) : value;
                }
            };
            columns[pn] = def.name;
            if (def.kind !== "column") {
                primaryKeys.push(pn);
                primaryKeyKinds.push(def.kind);
            }
            allKeys.push(pn);
        });
        if (!primaryKeyKinds.length) {
            throw new Error(`Schema Table '${name}' does not have a @SchemaBuilder.primaryKeyGenerated() or @SchemaBuilder.primaryKey() defined`);
        }
        else if (primaryKeyKinds.length > 1 && primaryKeyKinds.some(k => k === "primary-generated")) {
            throw new Error(`Schema Table '${name}': only a single primary key of type @SchemaBuilder.primaryKeyGenerated() is supported`);
        }
        const schema = {
            name,
            allKeys,
            primaryKeys,
            primaryKeyKind: primaryKeyKinds[0],
            qualifiedName,
            defs,
            columns,
            qualifiedColumns,
            mapper
        };
        return tableCache[name] = schema;
    }
    function col(kind, column) {
        return (target, propertyName) => {
            columnCache.push({
                target: target.constructor,
                propertyName,
                column: createColumn(column, propertyName),
                kind
            });
        };
    }
    function createColumn(column, name) {
        if (!column) {
            return { name };
        }
        if (typeof column === "string") {
            return { name: column };
        }
        if (!column.name) {
            column.name = name;
        }
        return column;
    }
    function primaryKeyGenerated(column) {
        return col("primary-generated", column);
    }
    SchemaBuilder.primaryKeyGenerated = primaryKeyGenerated;
    function primaryKey(column) {
        return col("primary", column);
    }
    SchemaBuilder.primaryKey = primaryKey;
    function column(column) {
        return col("column", column);
    }
    SchemaBuilder.column = column;
    function table(name) {
        return (target) => {
            const columns = columnCache.filter(c => c.target === target);
            target[TableNameKey] = name;
            tableDecoratedCache[name] = { name, columns };
        };
    }
    SchemaBuilder.table = table;
    function toColumn(i, qualifiedName) {
        return Object.assign({}, i.column, { propertyName: i.propertyName, kind: i.kind, qualifiedName });
    }
})(SchemaBuilder = exports.SchemaBuilder || (exports.SchemaBuilder = {}));
